from .download import download_zip, download_file
from pathlib import Path
down_path = Path(__file__).parent
xz_url = "addons_file"+"/"+down_path.name+"/"
download_zip(xz_url + "images.zip",down_path)
download_zip(xz_url + "lib.zip",down_path)

new_bl_info = {
    "name": "Magic Trails",
    "author": "Kolupsy",
    "version": ( 2, 3, 0 ),
    "blender": ( 2, 93, 0 ),
    "location": "View3D > Properties > Magic Trails",
    "description": "",
    "warning": "",
    "wiki_url": "",
    "category": "Animation",
}

import bpy
from bpy.props import *
import bpy.utils.previews

from . import operators, panel, simulation, trail_generator, utils

from .settings import trailsettings, settingutils, traildefaults, controller, trailmesh, follower

all_modules = [
    operators, panel, simulation, trail_generator, utils,
    trailsettings, settingutils, controller, trailmesh, follower, traildefaults
]

def register_classes( register, debug = False ):

    func = getattr( bpy.utils, 'register_class' if register else 'unregister_class' )
    for m in all_modules:
        if hasattr( m, 'get_classes' ):
            for c in m.get_classes( ):
                if debug:
                    try:
                        print( f'importing from "{m.__name__}": {c.__name__}.')
                    except:
                        print( f'Class in "{m.__name__}" has no name.')
                func( c )

def register( ):

    register_classes( True )

    panel.register_properties( True )
    traildefaults.register_properties( True )
    simulation.register_sim_handlers( True )

    utils.register_previews( )

    bpy.types.VIEW3D_MT_add.append( operators.add_trail_menu_item )
    bpy.types.VIEW3D_MT_object_parent.append( operators.follow_controller_menu_item )

def unregister( ):

    register_classes( False )
    panel.register_properties( False )
    traildefaults.register_properties( False )
    simulation.register_sim_handlers( False )

    utils.unregister_previews( )

    bpy.types.VIEW3D_MT_add.remove( operators.add_trail_menu_item )
    bpy.types.VIEW3D_MT_object_parent.remove( operators.follow_controller_menu_item )
