import bpy, platform, urllib
from bpy.types import Lattice, Panel, Scene
from bpy.props import *

from . import new_bl_info
from .utils import icon, word_wrap
from .operators import *
from .operators import draw_operator as op

def show_options( obj, context ):

    return context.scene.mtrail_panel_options or ( context.object == obj and obj.select_get( ))

class MagicTrailsPanel:

    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = "Magic Trails"
    bl_options = {"DEFAULT_CLOSED"}
    _tag = ''
    _icon = 'NONE'
    _icon_value = None

    @classmethod
    def poll( cls, context ):
        return len([ x for x in context.scene.objects if x.mtrail.type == cls._tag ])
    
    def draw_header( self, context ):
        layout = self.layout
        layout.label( text = "", icon = self._icon, icon_value = icon( self._icon_value ) if self._icon_value else 0 )
    
    def draw( self, context ):
        layout = self.layout
        objects = [ x for x in context.scene.objects if x.mtrail.type == self._tag ]
        for o in objects:
            o.mtrail.settings.draw_layout( layout, show_options = False )

class MAGICTRAILS_PT_Dependencies( MagicTrailsPanel, Panel ):

    bl_label = "Dependencies"
    bl_parent_id = 'VIEW3D_PT_MagicTrails'
    _tag = 'CONTROLLER'
    _icon = "RESTRICT_INSTANCED_OFF"

    def draw( self, context ):
        
        layout = self.layout
        show_options = False
        controllers = [ x for x in context.scene.objects if x.mtrail.is_controller ]
        drawn_objects = []
        for c in controllers:
            drawn_objects.append( c )
            children = c.mtrail.settings.children
            drawn_objects += children
            c.mtrail.settings.draw_layout( layout, show_options = show_options, draw_children = True )
        
        for o in [ x for x in context.scene.objects if not x in drawn_objects and x.mtrail.type in ('MESH', 'FOLLOWER' )]:
            drawn_objects.append( o )
            o.mtrail.settings.draw_layout( layout, show_options = show_options )

class MAGICTRAILS_PT_Controller( MagicTrailsPanel, Panel ):

    bl_label = "Controller"
    bl_parent_id = 'VIEW3D_PT_MagicTrails'
    _tag = 'CONTROLLER'
    _icon_value = 'TRAIL_CONTROLLER'

class MAGICTRAILS_PT_Mesh( MagicTrailsPanel, Panel ):

    bl_label = "Meshes"
    bl_parent_id = 'VIEW3D_PT_MagicTrails'
    _tag = 'MESH'
    _icon_value = 'TRAIL_MESH'

class MAGICTRAILS_PT_Follower( MagicTrailsPanel, Panel ):

    bl_label = "Follower"
    bl_parent_id = 'VIEW3D_PT_MagicTrails'
    _tag = 'FOLLOWER'
    _icon_value = 'TRAIL_FOLLOWER'

class MAGICTRAILS_PT_Properties_Object( Panel ):

    bl_label = 'Magic Trails'
    bl_idname = 'MAGICTRAILS_PT_Properties_Object'
    bl_space_type = "PROPERTIES"
    bl_region_type = "WINDOW"
    bl_context = 'object'
    bl_category = 'Magic Trails'
    bl_order = 0

    @classmethod
    def poll( cls, context ):
        return context.object
    
    def draw_header(self, context):
        
        layout = self.layout
        obj = context.object
        switch_icons = {
            'NONE' : ( 'NONE', icon( 'TRAIL' )),
            'CONTROLLER' : ( 'NONE', icon( 'TRAIL_CONTROLLER' )),
            'MESH' : ( 'NONE', icon( 'TRAIL_MESH' ) ),
            'FOLLOWER' : ( 'NONE', icon( 'TRAIL_FOLLOWER' ))
        }
        i, iv = switch_icons[ obj.mtrail.type ]
        layout.label( text = "", icon = i, icon_value = iv )
    
    def draw( self, context ):

        layout = self.layout
        obj = context.object
        obj.mtrail.draw_layout_default( layout )

class MAGICTRAILS_PT_Misc( MagicTrailsPanel, Panel ):

    bl_label = 'Misc'
    bl_parent_id = 'VIEW3D_PT_MagicTrails'
    bl_options = set()
    _icon = 'INFO'

    @classmethod
    def poll( cls, context ):
        return True
    
    def draw( self, context ):

        layout = self.layout
        region_width = context.region.width
        font_size = context.preferences.ui_styles[0].widget_label.points
        limit = min(( region_width / font_size ), 70)
        draw_info( layout, limit )

class MAGICTRAILS_PT_General( MagicTrailsPanel, Panel ):

    bl_label = "Magic Trails"
    bl_idname = 'VIEW3D_PT_MagicTrails'
    _icon_value = 'TRAIL'
    bl_options = set( )

    @classmethod
    def poll( cls, context ):

        return True

    def draw( self, context ):

        region_width = context.region.width
        font_size = context.preferences.ui_styles[0].widget_label.points
        limit = min(( region_width / font_size ), 70)

        layout = self.layout
        column = layout.column( align = True )

        if not context.preferences.filepaths.use_scripts_auto_execute:

            word_wrap( column, '请在首选项> 保存和加载中启用 “自动运行Python脚本”。', limit * 1.5 )
            op( column, MAGICTRAILS_OT_python_allow, text = "允许自动运行", icon = "PREFERENCES" )
            return

        box = column.box( )
        c = box.column( )
        c.scale_y = 1.5
        
        op( c, MAGICTRAILS_OT_new_trail, text = '添加新的轨迹', icon_value = icon( 'TRAIL_TIME' ))
        op( c, MAGICTRAILS_OT_follow_controller, icon_value = icon( 'TRAIL_FOLLOWER' ))

        if context.object:
            mt = context.object.mtrail
            if mt.enabled:
                column.separator( factor = 2 )
                column.label( text = 'Active Magic Trails Object:', icon = 'PRESET' )             
                mt.settings.draw_layout( column, show_options = True, draw_children = True )

def draw_info( layout, limit ):

    layout.label( text = "Tips:", icon = "INFO" )
    word_wrap( layout, "请记住烘焙您的轨迹,以锁定其动画以进行渲染。", limit * 1.5 )
    layout.separator( factor = 2 )

    name = new_bl_info[ "name" ]
    version = new_bl_info[ "version" ]
    version_compact = f'{version[0]}.{version[1]}.{version[2]}'
    bl_version = bpy.app.version_string
    system_info = platform.platform( )
    info = f'Version: ({version_compact}), Blender: ({bl_version}), System: ({system_info})'
    body = f'{urllib.parse.quote( info )}%0A%0ABug%20Report%20Checklist%20%28Check%20the%20System%20Console%20for%20errors%29%3A%0A-%20What%20happened%3A%0A%0A-%20What%20I%20expected%3A%0A%0A-%20How%20to%20reproduce%20this%20error%3A%0A%0A'
    subject = urllib.parse.quote( f'{name} Addon Feedback ({version_compact})' )
    url = f"mailto:kolupsy@gmail.com?subject={subject}&body={body}"
    layout.operator( "wm.url_open", text = "反馈/提交错误", icon = "URL" ).url = url

def get_classes( ):

    yield MAGICTRAILS_PT_General
    yield MAGICTRAILS_PT_Dependencies
    yield MAGICTRAILS_PT_Controller
    yield MAGICTRAILS_PT_Mesh
    yield MAGICTRAILS_PT_Follower
    yield MAGICTRAILS_PT_Misc
    yield MAGICTRAILS_PT_Properties_Object

def register_properties( register ):

    if register:
        Scene.mtrail_panel_options = BoolProperty( default = False, name = 'Show Trail Options' )
    else:
        del Scene.mtrail_panel_options