import stat
import bpy, os, re
from os.path import join
from math import acos
from pathlib import Path
from mathutils import Vector, Matrix
from .t3dn_bip import previews, settings as bip_settings

preview_collection = None

def get_python_filepath( ):

    return os.path.dirname( os.path.realpath( __file__ ))

def get_blend_file( filename ):

    s = r""
    blend_file = join( get_python_filepath( ), "lib", filename )
    return blend_file

def register_previews( ):

    global preview_collection
    bip_settings.WARNINGS = False
    preview_collection = previews.new( max_size = ( 256, 256 ) )
    bip_settings.WARNINGS = True

def unregister_previews( ):

    global preview_collection
    previews.remove( preview_collection )

def icon( name, file_type = '.bip' ):

    file = Path( __file__ ).resolve( )
    folder = file.parent / 'images'
    path = str( folder.joinpath( f'{name}{file_type}' ))
    image = preview_collection.load_safe( path, path, 'IMAGE' )
    return image.icon_id

def find_next_name( name, data ):

    if not name in data.keys( ): return name

    if len( name ) > 4 and name[-4] == "." and name[-3:].isdigit( ):
        number = int( name[-3:] ) +1
        new_name = name[:-3] + "{:03d}".format(number)
        return find_next_name( new_name, data )
    else:
        return name + ".001"

def word_wrap( layout, text, limit ):

    text = re.sub( r"\s+", " ", text )
    if text[0] == " ":
        text = text[1:]
    words = text.split( " " )
    accumulator = words[0]
    for w in words[1:]:
        if len(accumulator + " " + w) <= limit:
            accumulator += " " + w
        else: break
    row = layout.row( align = True )
    row.scale_y = 0.7

    row.label( text = accumulator )

    new_text = text[len( accumulator ):]
    if len( new_text ) > 0:
        word_wrap( layout, new_text, limit )

def get_trail_collection( ):

    coll_data = bpy.data.collections
    scene_coll = bpy.context.scene.collection
    coll_name = 'Magic Trails'
    if coll_name in coll_data.keys( ):
        coll = coll_data[ coll_name ]
        if not coll_name in scene_coll.children.keys( ):
            scene_coll.children.link( coll )
    else:
        coll = coll_data.new( coll_name )
        scene_coll.children.link( coll )
    
    return coll

def get_external_materials( ):

    with bpy.data.libraries.load( materials_file, link = False ) as ( data_from, data_to ):
        for m in ( x for x in data_from.materials if not x[0] == '_' ):
            yield m

def get_look_at_matrix( obmat, tarmat ):

    obvec = Vector(( 0, 0, 1 ))
    mat = Matrix( obmat ).to_3x3( )
    obvec = mat @ obvec
    if obvec.length == 0:
        obvec = ( 0, 0, 1 )

    obloc = obmat.translation
    tarvec = Vector(( tarmat.translation - obloc ))
    if tarvec.normalized( ).length == 0:
        tarvec = obvec
    
    raxis = obvec.cross( tarvec )
    rangle = obvec.dot( tarvec )
    rangle = acos( max( -1, min( 1, rangle )))
    rmat = Matrix.Rotation( rangle, 4, raxis )
    return rmat

materials_file = get_blend_file( "materials.blend" )